# -*- coding: utf-8 -*-
#
# (c) Copyright 2001-2007 Hewlett-Packard Development Company, L.P.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
# Author: Don Welch
#

import sys
from qt import *
from imagepropertiesdlg_base import ImagePropertiesDlg_base

class ImagePropertiesDlg(ImagePropertiesDlg_base):
    def __init__(self, filename, location, mimetype, size, exif_info={}, parent = None,name = None,modal = 0,fl = 0):
        ImagePropertiesDlg_base.__init__(self,parent,name,modal,fl)

        self.setCaption('Properties for ' + filename)
        self.FilenameText.setText('<b>' + filename + '</b>')
        self.LocationText.setText(location)
        self.MimeTypeText.setText(mimetype)
        self.SizeText.setText(size)
        if exif_info:
            for k in exif_info:
                kk = k.lower()
                if kk != "jpegthumbnail":
                    #print repr(k), repr(exif_info[k])
                    QListViewItem(self.EXifDataListView, k, str(exif_info[k]))







#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# (c) Copyright 2003-2008 Hewlett-Packard Development Company, L.P.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
# Author: Don Welch

from base.g import *
from base import utils

import os.path
import re
import glob

pat_prod_num = re.compile("""(\d+)""", re.I)

TYPE_UNKNOWN = 0
TYPE_STRING = 1
TYPE_STR = 1
TYPE_LIST = 2
TYPE_BOOL = 3
TYPE_INT = 4
TYPE_HEX = 5
TYPE_BITFIELD = 6
TYPE_URI = TYPE_STR # (7) not used (yet)

TECH_CLASSES = [
    "Undefined", # This will show an error (and its the default)
    "Unsupported", # This is for unsupported models, and it will not show an error
    "Postscript",
    "DJGenericVIP",
    #"PSB9100", not used on HPLIP
    "LJMono",
    "LJColor",
    "LJFastRaster",
    "LJJetReady",
    "DJ350",
    #"DJ400", not used on HPLIP
    "DJ540",
    "DJ600",
    "DJ6xx",
    "DJ6xxPhoto",
    "DJ630",
    #"DJ660", not used in HPLIP
    "DJ8xx",
    "DJ8x5",
    "DJ850",
    "DJ890",
    "DJ9xx",
    "DJ9xxVIP",
    "DJ3600",
    "DJ3320",
    "DJ4100",
    "AP2xxx",
    "AP21xx",
    "AP2560",
    "PSP100",
    "PSP470",
    "LJZjsMono",
    "LJZjsColor",
    "LJm1005",
    "QuickConnect",
    "DJ55xx",
    "OJProKx50",
]

TECH_CLASSES.sort()

TECH_CLASS_PDLS = {
    #"Undefined"    : '?',
    "Postscript"   : 'ps',
    "DJGenericVIP" : 'pcl3',
    #"PSB9100"      : 'pcl3',
    "LJMono"       : 'pcl3',
    "LJColor"      : 'pcl3',
    "LJFastRaster" : 'pclxl',
    "LJJetReady"   : 'pclxl',
    "DJ350"        : 'pcl3', 
    #"DJ400"        : 'pcl3',
    "DJ540"        : 'pcl3',
    "DJ600"        : 'pcl3',
    "DJ6xx"        : 'pcl3',
    "DJ6xxPhoto"   : 'pcl3',
    "DJ630"        : 'pcl3',
    #"DJ660"        : 'pcl3',
    "DJ8xx"        : 'pcl3',
    "DJ8x5"        : 'pcl3',
    "DJ850"        : 'pcl3',
    "DJ890"        : 'pcl3',
    "DJ9xx"        : 'pcl3',
    "DJ9xxVIP"     : 'pcl3',
    "DJ3600"       : 'lidil',
    "DJ3320"       : 'lidil',
    "DJ4100"       : 'lidil',
    "AP2xxx"       : 'pcl3',
    "AP21xx"       : 'pcl3',
    "AP2560"       : 'pcl3',
    "PSP100"       : 'pcl3',
    "PSP470"       : 'pcl3',
    "LJZjsMono"    : 'zjs',
    "LJZjsColor"   : 'zjs',
    "LJm1005"      : 'zxstream',
    "QuickConnect" : 'jpeg',
    "DJ55xx"       : 'pcl3',
    "OJProKx50"    : 'pcl3',
}


TECH_SUBCLASSES = [
    "LargeFormatSuperB",
    "LargeFormatA3",
    "CoverMedia", # 3425
    "FullBleed",
    "Duplex",
    "Normal",
    "Apollo2000",
    "Apollo2200",
    "Apollo2500",
    "NoPhotoMode",
    "NoPhotoBestHiresModes",
    "No1200dpiNoSensor",
    "NoFullBleed",
    "4x6FullBleed",
    "300dpiOnly",  # LaserJet 4L
    "GrayscaleOnly", # DJ540
]

TECH_SUBCLASSES.sort()


# Items will be capitalized unless in this dict
MODEL_UI_REPLACEMENTS = {'laserjet'   : 'LaserJet',
                          'psc'        : 'PSC',
                          'officejet'  : 'Officejet',
                          'deskjet'    : 'Deskjet',
                          'hp'         : 'HP',
                          'business'   : 'Business',
                          'inkjet'     : 'Inkjet',
                          'photosmart' : 'Photosmart',
                          'color'      : 'Color',
                          'series'     : 'series',
                          'printer'    : 'Printer',
                          'mfp'        : 'MFP',
                          'mopier'     : 'Mopier',
                          'pro'        : 'Pro',
                          'apollo'     : 'Apollo',
                        }


def normalizeModelUIName(model):
    ml = model.lower().strip()

    if 'apollo' in ml:
        z = ml.replace('_', ' ')
    else:
        if ml.startswith("hp"):
            z = ml[3:].replace('_', ' ')
        else:
            z = ml.replace('_', ' ')

    y = []
    for x in z.split():
        if pat_prod_num.search(x): # don't cap items like cp1700dn
            y.append(x)
        else:
            y.append(MODEL_UI_REPLACEMENTS.get(x, x.capitalize()))

    if 'apollo' in ml:
        return ' '.join(y)
    else:
        return "HP " + ' '.join(y)


def normalizeModelName(model):
    return utils.xstrip(model.replace(' ', '_').replace('__', '_').replace('~','').replace('/', '_'), '_')


class ModelData:
    def __init__(self, root_path=None):
        if root_path is None:
            self.root_path = prop.models_dir
        else:
            self.root_path = root_path

        self.__cache = {}
        self.reset_includes()
        self.sec = re.compile(r'^\[(.*)\]')
        self.inc = re.compile(r'^\%include (.*)', re.IGNORECASE)
        self.inc_line = re.compile(r'^\%(.*)\%')
        self.eq = re.compile(r'^([^=]+)=(.*)')

        files = [(os.path.join(self.root_path, "models.dat"), 
                  os.path.join(self.root_path, "unreleased", "unreleased.dat")),
                 (os.path.join(os.getcwd(), 'data', 'models', 'models.dat'), 
                  os.path.join(os.getcwd(), 'data', 'models', 'unreleased', 'unreleased.dat'))]

        for self.released_dat, self.unreleased_dat in files:
            if os.path.exists(self.released_dat):
                break
            
        else:
            self.released_dat, self.unreleased_dat = None, None
            log.error("Unable to locate models.dat file")

        self.FIELD_TYPES = {
            # Static model query data (from models.dat)
            'align-type' : TYPE_INT,
            'clean-type' : TYPE_INT,
            'color-cal-type' : TYPE_INT,
            'copy-type' : TYPE_INT,
            'embedded-server-type' : TYPE_INT,
            'fax-type' : TYPE_INT,
            'fw-download' : TYPE_BOOL,
            'icon' : TYPE_STR,
            'io-mfp-mode' : TYPE_INT,
            'io-mode' : TYPE_INT,
            'io-support' : TYPE_BITFIELD,
            'monitor-type' : TYPE_INT,
            'linefeed-cal-type' : TYPE_INT,
            'panel-check-type' : TYPE_INT,
            'pcard-type' : TYPE_INT,
            'plugin' : TYPE_INT,
            'power-settings': TYPE_INT,
            'pq-diag-type' : TYPE_INT,
            'r-type' : TYPE_INT,
            'scan-style' : TYPE_INT,
            'scan-type' : TYPE_INT,
            'status-battery-check' : TYPE_INT,
            'status-dynamic-counters' : TYPE_INT,
            'status-type' : TYPE_INT,
            'support-released' : TYPE_BOOL,
            'support-type' : TYPE_INT,
            'support-ver' : TYPE_STR,
            'tech-class' : TYPE_LIST,
            'tech-subclass' : TYPE_LIST,
            'tech-type' : TYPE_INT,
            'usb-pid' : TYPE_HEX,
            'usb-vid' : TYPE_HEX,
            'job-storage' : TYPE_INT,

            # Dynamic model data (from device query)
            'dev-file' : TYPE_STR,
            'fax-uri' : TYPE_STR,
            'scan-uri' : TYPE_STR,
            'is-hp' : TYPE_BOOL,
            'host' : TYPE_STR,
            'status-desc' : TYPE_STR,
            'cups-printer' : TYPE_STR,
            'serial' : TYPE_STR,
            'error-state' : TYPE_INT,
            'device-state' : TYPE_INT,
            'panel' : TYPE_INT,
            'device-uri' : TYPE_STR,
            'panel-line1' : TYPE_STR,
            'panel-line2' : TYPE_STR,
            'back-end' : TYPE_STR,
            'port' : TYPE_INT,
            'deviceid' : TYPE_STR,
            'cups-uri' : TYPE_STR,
            'status-code' : TYPE_INT,
            'rs' : TYPE_STR,
            'rr' : TYPE_STR,
            'rg' : TYPE_STR,
            'r' : TYPE_INT,
            'duplexer' : TYPE_INT,
            'supply-door' : TYPE_INT,
            'revision' : TYPE_INT,
            'media-path' : TYPE_INT,
            'top-door' : TYPE_BOOL,
            'photo-tray' : TYPE_BOOL,
            }

        self.RE_FIELD_TYPES = {
            re.compile('^r(\d+)-agent(\d+)-kind', re.IGNORECASE) : TYPE_INT,
            re.compile('^r(\d+)-agent(\d+)-type', re.IGNORECASE) : TYPE_INT,
            re.compile('^r(\d+)-agent(\d+)-sku', re.IGNORECASE) : TYPE_STR,
            re.compile('^agent(\d+)-desc', re.IGNORECASE) : TYPE_STR,
            re.compile('^agent(\d+)-virgin', re.IGNORECASE) : TYPE_BOOL,
            re.compile('^agent(\d+)-dvc', re.IGNORECASE) : TYPE_INT,
            re.compile('^agent(\d+)-kind', re.IGNORECASE) : TYPE_INT,
            re.compile('^agent(\d+)-type', re.IGNORECASE) : TYPE_INT,
            re.compile('^agent(\d+)-id', re.IGNORECASE) : TYPE_INT,
            re.compile('^agent(\d+)-hp-ink', re.IGNORECASE) : TYPE_BOOL,
            re.compile('^agent(\d+)-health-desc', re.IGNORECASE) : TYPE_STR,
            re.compile('^agent(\d+)-health$', re.IGNORECASE) : TYPE_INT,
            re.compile('^agent(\d+)-known', re.IGNORECASE) : TYPE_BOOL,
            re.compile('^agent(\d+)-level', re.IGNORECASE) : TYPE_INT,
            re.compile('^agent(\d+)-ack', re.IGNORECASE) : TYPE_BOOL,
            re.compile('^agent(\d+)-sku', re.IGNORECASE) : TYPE_STR,
            re.compile('^in-tray(\d+)', re.IGNORECASE) : TYPE_BOOL,
            re.compile('^out-tray(\d+)', re.IGNORECASE) : TYPE_BOOL,
            re.compile('model(\d+)', re.IGNORECASE) : TYPE_STR,
            }

        self.TYPE_CACHE = {}


    def read_all_files(self, unreleased=True):
        if os.path.exists(self.released_dat):
            self.read_section(self.released_dat)

            if self.unreleased_dat is not None and os.path.exists(self.unreleased_dat):
                self.read_section(self.unreleased_dat )

        return self.__cache


    def read_section(self, filename, section=None, is_include=False): # section==None, read all sections
        found, in_section = False, False

        if section is not None:
            section = section.lower()

            if is_include:
                log.debug("Searching for include [%s] in file %s" % (section, filename))
            else:
                log.debug("Searching for section [%s] in file %s" % (section, filename))

        if is_include:
            cache = self.__includes
        else:
            cache = self.__cache

        try:
            fd = file(filename)
        except IOError, e:
            log.error("I/O Error: %s (%s)" % (filename, e.strerror))
            return False

        while True:
            line = fd.readline()

            if not line:
                break

            if line[0] in ('#', ';'):
                continue

            if line[0] == '[':
                if in_section and section is not None:
                    break

                match = self.sec.search(line)

                if match is not None:
                    in_section = True

                    read_section = match.group(1).lower()

                    if section is not None:
                        found = in_section = (read_section == section)

                    if in_section:
                        if section is not None:
                            log.debug("Found section [%s] in file %s" % (read_section, filename))

                        cache[read_section] = {}

                continue

            if line[0] == '%':
                match = self.inc.match(line)

                if match is not None:
                    inc_file = match.group(1)
                    log.debug("Found include file directive: %%include %s" % inc_file)
                    self.__include_files.append(os.path.join(os.path.dirname(filename), inc_file))
                    continue

                if in_section:
                    match = self.inc_line.match(line)

                    if match is not None: 
                        inc_sect = match.group(1)
                        log.debug("Found include directive %%%s%%" % inc_sect)

                        try:
                            self.__includes[inc_sect]
                        except KeyError:
                            for inc in self.__include_files:

                                if self.read_section(inc, inc_sect, True):
                                    break
                            else:
                                log.error("Include %%%s%% not found." % inc_sect)

            if in_section:
                match = self.eq.search(line)

                if match is not None:
                    key = match.group(1)
                    value = match.group(2)
                    value = self.convert_data(key, value)
                    cache[read_section][key] = value

        fd.close()
        return found


    def reset_includes(self):
        self.__include_files = []
        self.__includes = {}


    def __getitem__(self, model):
        model = model.lower()

        try:
            return self.__cache[model]
        except:
            log.debug("Cache miss: %s" % model)

            log.debug("Reading file: %s" % self.released_dat)

            if self.read_section(self.released_dat, model):
                return self.__cache[model]

            if self.unreleased_dat is not None and os.path.exists(self.unreleased_dat):
                log.debug("Reading file: %s" % self.unreleased_dat)

                if self.read_section(self.unreleased_dat, model):
                    return self.__cache[model]

            return {}


    def all_models(self):
        return self.__cache


    def get_data_type(self, key):
        try:
            return self.FIELD_TYPES[key]
        except KeyError:
            try:
                return self.TYPE_CACHE[key]
            except KeyError:
                for pat, typ in self.RE_FIELD_TYPES.items():
                    match = pat.match(key)
                    if match is not None:
                        self.TYPE_CACHE[key] = typ
                        return typ

        log.warn("get_data_type(): Defaulted to TYPE_STR for key %s" % key)
        return TYPE_STR


    def convert_data(self, key, value, typ=None):
        if typ is None:
            typ = self.get_data_type(key)

        if  typ in (TYPE_BITFIELD, TYPE_INT):
            value = int(value)

        elif typ == TYPE_BOOL:
            value = utils.to_bool(value)
            #value = int(value)

        elif typ == TYPE_LIST:
            value = [x for x in value.split(',') if x]

        return value



# -*- coding: utf-8 -*-
#
# (c) Copyright 2003-2007 Hewlett-Packard Development Company, L.P.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
# Author: Don Welch
#

# string_table := { 'string_id' : 'short', 'long' ), ... }

# string_id's for error codes are the string form of the error code
# Strings that need localization use  (self.__tr'string' ) form.
# Strings that refer to other strings, use '%reference%' form.
# Blank strings use '' form.

class StringTable:
    def __init__(self):

        self.string_table = {

            '0' :               (self.__tr('Unknown'),
                                 ''),

            'unknown'   :       (self.__tr('Unknown'),
                                 ''),

            'try_again' :       ('',
                                 self.__tr('Please correct the problem and try again.')),

            'press_continue' :  ('',
                                 self.__tr('Please correct the problem and press continue on the printer.')),

            '500' :             (self.__tr('Started a print job'),
                                 ''),

            '501' :             (self.__tr('Print job has completed'),
                                 ''),

            '600' :             (self.__tr('Started a fax job'),
                                 ''),
                                 
            '601' :             (self.__tr('Fax job is ready for send'),
                                 ''),

            '700' :             (self.__tr('Printer queue stopped'),
                                 ''),
                                 
            '701' :             (self.__tr('Printer queue started'),
                                 ''),

            '702' :             (self.__tr('Printer is rejecting jobs'),
                                 ''),

            '703' :             (self.__tr('Printer is accepting jobs'),
                                 ''),

            '704' :             (self.__tr('Printer set as default'),
                                 ''),
                                 
            '800' :             (self.__tr('Fax queue stopped'),
                                 ''),
                                 
            '801' :             (self.__tr('Fax queue started'),
                                 ''),

            '802' :             (self.__tr('Fax is rejecting jobs'),
                                 ''),

            '803' :             (self.__tr('Fax is accepting jobs'),
                                 ''),

            '804' :             (self.__tr('Fax set as default'),
                                 ''),

            '1000' :            (self.__tr('Idle'),
                                  ''),

            '1001' :            (self.__tr('Busy'),
                                 ''),

            '1002' :            (self.__tr('Print job is continuing'),
                                  ''),

            '1003' :            (self.__tr('Turning off'),
                                  ''),

            '1004' :            (self.__tr('Report printing'),
                                  ''),

            '1005' :            (self.__tr('Canceling'),
                                  ''),

            '1006' :            ('%5002%',
                                '%try_again%'),

            '1007' :            (self.__tr('Waiting for ink to dry'),
                                 ''),

            '1008' :            (self.__tr('Pen change'),
                                 ''),

            '1009' :            (self.__tr('The printer is out of paper'),
                                 self.__tr('Please load more paper and follow the instructions on the front panel (if any) to continue printing.')),

            '1010' :            (self.__tr('Banner eject needed'),
                                 ''),

            '1011' :            (self.__tr('Banner mismatch'),
                                 '%try_again%'),

            '1012' :            (self.__tr('Photo mismatch'),
                                 '%try_again%'),

            '1013' :            (self.__tr('Duplex mismatch'),
                                 '%try_again'),

            '1014' :            (self.__tr('Paper or cartridge carriage jammed'),
                                 self.__tr('Please clear the jam and press continue on the printer.')),

            '1015' :            ('%1014%',
                                 '%1014%'),

            '1016' :            ('%1014%',
                                 '%1014%'),

            '1017' :            (self.__tr('There is a problem with a print cartridge'),
                                 '%press_continue%'),

            '1018' :            ('%unknown_error%',
                                  '%try_again%'),

            '1019' :            (self.__tr('Powering down'),
                                 ''),

            '1020' :            (self.__tr('Front panel test'),
                                 ''),

            '1021' :            (self.__tr('Clean out tray missing'),
                                 '%try_again%'),

            '1022' :            (self.__tr('Output bin full'),
                                 '%try_again%'),

            '1023' :            (self.__tr('Media size mismatch'),
                                 '%try_again%'),

            '1024' :            (self.__tr('Duplexer is jammed'),
                                 '%1014%'),

            '1025' :            ('%1014%',
                                 '%1014%'),

            '1026' :            (self.__tr('An ink cartridge is out of ink'),
                                 '%try_again%'),

            '1027' :            (self.__tr('Internal device error'),
                                 '%try_again%'),

            '1028' :            ('%1014%',
                                 '%1014%'),

            '1029' :            (self.__tr('Second tray missing'),
                                 '%try_again%'),

            '1030' :            (self.__tr('Duplexer missing'),
                                 '%try_again%'),

            '1031' :            (self.__tr('Rear tray missing'),
                                 '%try_again%'),

            '1032' :            (self.__tr('Cartridge not latched'),
                                 '%try_again%'),

            '1033' :            (self.__tr('Battery very low'),
                                 '%try_again%'),

            '1034' :            ('%1017%',
                                 '%try_again%'),

            '1035' :            (self.__tr('Output tray closed'),
                                 '%try_again%'),

            '1036' :            (self.__tr('Manual feed blocked'),
                                 '%1014%'),

            '1037' :            (self.__tr('Rear feed blocked'),
                                 '%1014%'),

            '1038' :            (self.__tr('Second tray out of paper'),
                                 '%1009%'),

            '1039' :            (self.__tr('Input tray locked'),
                                 '%try_again%'),

            '1040' :            (self.__tr('Non-HP ink'),
                                 '%try_again%'),

            '1041' :            (self.__tr('Pen calibration needs resume'),
                                 '%press_continue%'),

            '1042' :            (self.__tr('Media type mismatch'),
                                 '%try_again%'),

            '1043' :            (self.__tr('Custom media mismatch'),
                                 '%try_again%'),

            '1044' :            (self.__tr('Pen cleaning in progress'),
                                 ''),

            '1045' :            (self.__tr('Pen checking in progress'),
                                 ''),

            '1501' :            (self.__tr('Black cartridge is low on ink'),
                                 ''),

            '1502' :            (self.__tr('Tri-color cartridge is low on ink'),
                                 ''),

            '1503' :            (self.__tr('Photo cartridge is low on ink'),
                                 ''),

            '1504' :            (self.__tr('Cyan cartridge is low on ink'),
                                 ''),

            '1505' :            (self.__tr('Magenta cartridge is low on ink'),
                                 ''),

            '1506' :            (self.__tr('Yellow cartridge is low on ink'),
                                 ''),

            '1507' :            (self.__tr('Photo cyan cartridge is low on ink'),
                                 ''),

            '1508' :            (self.__tr('Photo magenta cartridge is low on ink'),
                                 ''),

            '1509' :            (self.__tr('Photo yellow cartridge is low on ink'),
                                 ''),

            '1510' :            (self.__tr('Photo gray cartridge is low on ink'),
                                 ''),

            '1511' :            (self.__tr('Photo blue cartridge is low on ink'),
                                 ''),

            '1601' :            (self.__tr('Black cartridge is low on toner'),
                                 ''),

            '1604' :            (self.__tr('Cyan cartridge is low on toner'),
                                 ''),

            '1605' :            (self.__tr('Magenta cartridge is low on toner'),
                                 ''),

            '1606' :            (self.__tr('Yellow cartridge is low on toner'),
                                 ''),

            '1800' :            (self.__tr('Warming up'),
                                 ''),

            '1801' :            (self.__tr('Low paper'),
                                 ''),

            '1802' :            (self.__tr('Door open'),
                                 '%try_again%'),

            '1803' :            (self.__tr('Offline'),
                                 ''),

            '1804' :            (self.__tr('Low toner'),
                                 ''),

            '1805' :            (self.__tr('No toner'),
                                 '%try_again%'),

            '1806' :            (self.__tr('Service request'),
                                 '%try_again%'),

            '1807' :            (self.__tr('Fuser error'),
                                 '%try_again%'),

            '1900' :            (self.__tr('Unsupported printer model'),
                                 ''),

            '2000' :            (self.__tr('Scan job started'),
                                 ''),

            '2001' :            (self.__tr('Scan job completed'),
                                 ''),

            '2002' :            (self.__tr('Scan job failed'),
                                 '%try_again%'),

            '3000' :            (self.__tr('Fax job started'),
                                 ''),

            '3001' :            (self.__tr('Fax job complete'),
                                 ''),

            '3002' :            (self.__tr('Fax job failed'),
                                 '%try_again%'),

            '3003' :            (self.__tr('Fax job canceled'),
                                 ''),

            '3004' :            (self.__tr('Fax send job continuing'),
                                 ''),

            '3005' :            (self.__tr('Fax receive job continuing'),
                                 ''),

            '4000' :            (self.__tr('Copy job started'),
                                 ''),

            '4001' :            (self.__tr('Copy job complete'),
                                 ''),

            '4002' :            (self.__tr('Copy job failed'),
                                 '%try_again%'),

            '4003' :            (self.__tr('Copy job canceled'),
                                 ''),

            '5002' :            (self.__tr('Device is busy, powered down, or unplugged'),
                                 '%5012%'),

            '5004' :            (self.__tr('Invalid device URI'),
                                 '%5012%'),

            '5012' :            (self.__tr('Device communication error'),
                                 '%try_again%'),

            '5021' :            (self.__tr('Device is busy'),
                                 ''),

            '5026' :            (self.__tr("Device status not available"),
                                 ''),

            '5031' :            ('%5021%',
                                 ''),

            '5034' :            (self.__tr('Device does not support requested operation'),
                                 '%try_again%'),

            '6000' :            (self.__tr('Photocard unload started'),
                                 ''),

            '6001' :            (self.__tr('Photocard unload ended'),
                                 ''),

            '6002' :            (self.__tr('Photocard unload failed'),
                                 self.__tr('Make sure photocard is inserted properly and try again.')),

            '6003' :            (self.__tr('Unable to mount photocard on device'),
                                 '%6002%'),

            '6004' :            (self.__tr('Photocard unloaded successfully'),
                                 ''),

            'unknown_error' :   (self.__tr('Unknown error'),
                                 ''),

            'print' :           (self.__tr('Print'),
                                 ''),

            'scan' :            (self.__tr('Scan'),
                                 ''),

            'send_fax' :        (self.__tr('Send fax'),
                                 ''),

            'make_copies' :     (self.__tr('Make copies'),
                                 ''),

            'access_photo_cards' :    (self.__tr('Access photo cards'),
                                       ''),

            'agent_invalid_invalid' : (self.__tr('Invalid/missing'),
                                       ''),

            'agent_invalid_supply' :  (self.__tr('Invalid/missing ink cartridge'),
                                       ''),

            'agent_invalid_cartridge':(self.__tr('Invalid/missing cartridge'),
                                       ''),

            'agent_invalid_head' :    (self.__tr('Invalid/missing print head'),
                                       ''),

            'agent_unknown_unknown' : ('%unknown%',
                                       ''),

            'agent_unspecified_battery' : ('Battery',
                                           ''),

            'agent_black_head' :      (self.__tr('Black print head'),
                                       ''),

            'agent_black_supply' :    (self.__tr('Black ink cartridge'),
                                       ''),

            'agent_black_cartridge' : (self.__tr('Black cartridge'),
                                       ''),

            'agent_cmy_head' :        (self.__tr('Tri-color print head'),
                                       ''),

            'agent_cmy_supply' :      (self.__tr('Tri-color ink cartridge'),
                                       ''),

            'agent_cmy_cartridge' :   (self.__tr('Tri-color cartridge'),
                                       ''),

            'agent_kcm_head' :        (self.__tr('Photo print head'),
                                       ''),

            'agent_kcm_supply' :      (self.__tr('Photo ink cartridge'),
                                       ''),

            'agent_kcm_cartridge' :   (self.__tr('Photo cartridge'),
                                       ''),

            'agent_cyan_head' :       (self.__tr('Cyan print head'),
                                       ''),

            'agent_cyan_supply' :     (self.__tr('Cyan ink cartridge'),
                                       ''),

            'agent_cyan_cartridge' :  (self.__tr('Cyan cartridge'),
                                       ''),

            'agent_magenta_head' :    (self.__tr('Magenta print head'),
                                       ''),

            'agent_magenta_supply' :  (self.__tr('Magenta ink cartridge'),
                                       ''),

            'agent_magenta_cartridge':(self.__tr('Magenta cartridge'),
                                       ''),

            'agent_yellow_head' :     (self.__tr('Yellow print head'),
                                       ''),

            'agent_yellow_supply' :   (self.__tr('Yellow ink cartridge'),
                                       ''),

            'agent_yellow_cartridge': (self.__tr('Yellow cartridge'),
                                       ''),

            'agent_photo_cyan_head' :       (self.__tr('Photo cyan print head'),
                                             ''),

            'agent_photo_cyan_supply' :     (self.__tr('Photo cyan ink cartridge'),
                                             ''),

            'agent_photo_cyan_cartridge' :  (self.__tr('Photo cyan cartridge'),
                                             ''),

            'agent_photo_magenta_head' :    (self.__tr('Photo magenta print head'),
                                             ''),

            'agent_photo_magenta_supply' :  (self.__tr('Photo magenta ink cartridge'),
                                             ''),

            'agent_photo_magenta_cartridge':(self.__tr('Photo magenta cartridge'),
                                             ''),

            'agent_photo_yellow_head' :     (self.__tr('Photo yellow print head'),
                                             ''),

            'agent_photo_yellow_supply' :   (self.__tr('Photo yellow ink cartridge'),
                                             ''),

            'agent_photo_yellow_cartridge': (self.__tr('Photo yellow cartridge'),
                                             ''),

            'agent_photo_gray_head' :       (self.__tr('Photo gray print head'),
                                             ''),

            'agent_photo_gray_supply' :     (self.__tr('Photo gray ink cartridge'),
                                             ''),

            'agent_photo_gray_cartridge' :  (self.__tr('Photo gray cartridge'),
                                             ''),

            'agent_photo_blue_head' :       (self.__tr('Photo blue print head'),
                                             ''),

            'agent_photo_blue_supply' :     (self.__tr('Photo blue ink cartridge'),
                                             ''),

            'agent_photo_blue_cartridge' :  (self.__tr('Photo blue cartridge'),
                                             ''),

            'agent_kcmy_cm_head' :          (self.__tr('Print head'),
                                             ''),

            'agent_photo_cyan_and_photo_magenta_head' : (self.__tr('Photo magenta and photo cyan print head'),
                                                         ''),

            'agent_yellow_and_magenta_head' :           (self.__tr('Magenta and yellow print head'), 
                                                         '' ),

            'agent_cyan_and_black_head' :               (self.__tr('Black and cyan print head'),
                                                         '' ),

            'agent_light_gray_and_photo_black_head' :    (self.__tr('Light gray and photo black print head'),
                                                          '' ),

            'agent_light_gray_supply' :                 (self.__tr('Light gray ink cartridge'), # LG
                                                         '' ),

            'agent_medium_gray_supply' :                (self.__tr('Medium gray ink cartridge'), 
                                                         '' ),

            'agent_photo_gray_supply' :                 (self.__tr('Photo black ink cartridge'), # PK
                                                         '' ),

            'agent_cyan_and_magenta_head' :             (self.__tr('Cyan and magenta print head'),
                                                          ''),

            'agent_black_and_yellow_head' :             (self.__tr('Black and yellow print head'),
                                                          ''),

            'agent_black_toner' :           (self.__tr('Black toner cartridge'),
                                             ''),

            'agent_cyan_toner' :            (self.__tr('Cyan toner cartridge'),
                                             ''),

            'agent_magenta_toner' :         (self.__tr('Magenta toner cartridge'),
                                             ''),

            'agent_yellow_toner' :          (self.__tr('Yellow toner cartridge'),
                                             ''),

            'agent_unspecified_maint_kit' : (self.__tr('Maintenance kit (fuser)'),
                                             ''),

            'agent_unspecified_adf_kit'   : (self.__tr('Document feeder (ADF) kit'),
                                             ''),

            'agent_unspecified_drum_kit'   : (self.__tr('Drum maintenance kit'),
                                              ''),

            'agent_unspecified_transfer_kit'   : (self.__tr('Image transfer kit'),
                                                  ''),

            'agent_health_unknown'     : ('Unknown',
                                          ''),

            'agent_health_ok'          : (self.__tr('Good/OK'),
                                           ''),

            'agent_health_fair_moderate' : (self.__tr('Fair/Moderate'),
                                            ''),

            'agent_health_misinstalled': (self.__tr('Not installed'),
                                          ''),

            'agent_health_incorrect'   : (self.__tr('Incorrect'),
                                          ''),

            'agent_health_failed'      : (self.__tr('Failed'),
                                          ''),

            'agent_health_overtemp'      : (self.__tr('Overheated'),
                                            ''),

            'agent_health_discharging'      : (self.__tr('Discharging'),
                                               ''),

            'agent_health_charging'      : (self.__tr('Charging'),
                                            ''),

            'agent_level_unknown'      : ('%unknown%',
                                          ''),

            'agent_level_low'          : (self.__tr('Low'),
                                          ''),

            'agent_level_out'          : (self.__tr('Very low'),
                                          ''),

#            'email_test_subject'      : (self.__tr('HPLIP: Email alert test message'),
#                                         ''),
#
#            'email_test_message'      : (self.__tr('This email is to test the functionality of HPLIP email alerts'),
#                                         ''),
#
#            'email_alert_subject'      : (self.__tr('HPLIP: Error/alert on device: '),
#                                          ''),

        }

    def __tr(self,s,c = None):
        return s







# -*- coding: utf-8 -*-
#
# (c) Copyright 2003-2008 Hewlett-Packard Development Company, L.P.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
# Author: Don Welch
#

from base.g import *
from installer import core_install
from base import device
        
from qt import *
from pluginform2_base import PluginForm2_base

class PluginForm2(PluginForm2_base):
    def __init__(self,parent = None,name = None,modal = 0,fl = 0):
        PluginForm2_base.__init__(self,parent,name,modal,fl)
        self.path = None
        #self.version = sys_cfg.hplip.version
        self.version = '.'.join(sys_cfg.hplip.version.split('.')[:3])
        #print self.version
        self.bg = self.pathLineEdit.paletteBackgroundColor()
        
        self.titleTextLabel.setFont(QFont('Helvetica', 16))
        
        self.sourceGroup.emit(SIGNAL("clicked(int)"), (0,))
        
    def sourceGroup_clicked(self, item):
        self.pathLineEdit.setEnabled(item == 1)
        self.browsePushButton.setEnabled(item == 1)
        
        #if item == 1 and 
        #    self.actionPushButton.setText(self.__tr("Download and Install"))
            
        if item == 0: # download
            QToolTip.remove(self.pathLineEdit)
            self.actionPushButton.setText(self.__tr("Download and Install"))
            self.actionPushButton.setEnabled(True)
            self.path = None
        else: # path
            self.path = unicode(self.pathLineEdit.text())
            self.pathLineEdit.emit(SIGNAL("textChanged(const QString&)"), (self.path,)) 
            
            if self.path.startswith(u"http://"):
                self.actionPushButton.setText(self.__tr("Download and Install"))
            else:   
                self.actionPushButton.setText(self.__tr("Copy and Install"))
            
            
    def browsePushButton_clicked(self):
        workingDirectory = user_cfg.last_used.working_dir

        if not workingDirectory or not os.path.exists(workingDirectory):
            workingDirectory = os.path.expanduser("~")

        log.debug("workingDirectory: %s" % workingDirectory)

        dlg = QFileDialog(workingDirectory, self.__tr("HPLIP %s Plug-in (*.run)" % 
            self.version), None, None, True)

        dlg.setCaption("openfile")
        dlg.setMode(QFileDialog.ExistingFile)
        dlg.show()

        if dlg.exec_loop() == QDialog.Accepted:
            results = dlg.selectedFile()
            workingDirectory = unicode(dlg.dir().absPath())
            log.debug("results: %s" % results)
            log.debug("workingDirectory: %s" % workingDirectory)

            user_cfg.last_used.working_dir = workingDirectory

            if results:
                self.path = unicode(results)
                self.pathLineEdit.setText(self.path)
                    
                    
    def pathLineEdit_textChanged(self, path):
        path, ok = unicode(path), True
        
        if not path.startswith(u'http://'):
            self.actionPushButton.setText(self.__tr("Copy and Install"))
            
            if not path or not os.path.exists(path):
                #log.error("File '%s' not found." % path)
                QToolTip.add(self.pathLineEdit, self.__tr('File not found.'))
                ok = False
                
            elif os.path.basename(path) != u'hplip-%s-plugin.run' % self.version:
                log.error("Incorrect file: %s (should be: %s)" % (path, 'hplip-%s-plugin.run' % self.version))
                QToolTip.add(self.pathLineEdit, self.__tr("Incorrect file. Must be '%1'")\
                    .arg('hplip-%s-plugin.run' % self.version))
                    
                ok = False
        
        else:
            self.actionPushButton.setText(self.__tr("Download and Install"))
            
        self.actionPushButton.setEnabled(ok)
        
        if not ok:
            self.pathLineEdit.setPaletteBackgroundColor(QColor(0xff, 0x99, 0x99))
            
        else:
            QToolTip.remove(self.pathLineEdit)
            self.pathLineEdit.setPaletteBackgroundColor(self.bg)
            self.path = path
            
            
    def actionPushButton_clicked(self):
        core = core_install.CoreInstall()
        core.set_plugin_version() #self.version)

        if self.path is None: # download
            # read plugin.conf (local or on sf.net) to get plugin_path (http://)
            plugin_conf_url = core.get_plugin_conf_url()
            
            if plugin_conf_url.startswith('file://'):
                #tui.header("COPY CONFIGURATION")
                pass
            else:
                pass #tui.header("DOWNLOAD CONFIGURATION")
                
                log.info("Checking for network connection...")
                ok = core.check_network_connection()
                
                if not ok:
                    log.error("Network connection not detected.")
                    #sys.exit(1)
                    self.FailureUI(self.__tr("Network connection not detected."))
                    #return
                    self.close()
                    return
                
            
            log.info("Downloading configuration file from: %s" % plugin_conf_url)
            #pm = tui.ProgressMeter("Downloading configuration:")
            
            self.path, size, checksum, timestamp, ok = core.get_plugin_info(plugin_conf_url, 
                self.plugin_download_callback)
            
            print self.path, size, checksum, timestamp, ok
            
            #sys.exit(1)
            
            if not self.path.startswith('http://') and not self.path.startswith('file://'):
                self.path = 'file://' + self.path
        
        
        else: # path
            if not self.path.startswith('http://'):
                self.path = 'file://' + self.path
                
            size, checksum, timestamp = 0, '', 0
            
        
        if self.path.startswith('file://'):
            #tui.header("COPY PLUGIN")
            pass
        else:
            #tui.header("DOWNLOAD PLUGIN")
            
            log.info("Checking for network connection...")
            ok = core.check_network_connection()
            
            if not ok:
                log.error("Network connection not detected.")
                self.FailureUI(self.__tr("Network connection not detected."))
                self.close()
                return
    
        log.info("Downloading plug-in from: %s" % self.path)
        #pm = tui.ProgressMeter("Downloading plug-in:")
        
        ok, local_plugin = core.download_plugin(self.path, size, checksum, timestamp, 
            self.plugin_download_callback)
            
        #print
        
        if not ok:
            log.error("Plug-in download failed: %s" % local_plugin)
            self.FailureUI(self.__tr("Plug-in download failed."))
            #sys.exit(1)
            self.close()
            return
        
        #tui.header("INSTALLING PLUG-IN")
        
        if not core.run_plugin(GUI_MODE, self.plugin_install_callback):
            self.FailureUI(self.__tr("Plug-in install failed."))
            self.close()
            return
        
        cups_devices = device.getSupportedCUPSDevices(['hp']) #, 'hpfax'])
        #print cups_devices
        
        for dev in cups_devices:
            mq = device.queryModelByURI(dev)
            
            if mq.get('fw-download', False):
                
                # Download firmware if needed
                log.info(log.bold("\nDownloading firmware to device %s..." % dev))
                try:
                    d = device.Device(dev)
                except Error:
                    log.error("Error opening device.")
                    #sys.exit(1)
                    continue

                if d.downloadFirmware():
                    log.info("Firmware download successful.\n")

                d.close()
        
        
        self.SuccessUI("Plug-in install successful.")
        self.close()
        
    
    def FailureUI(self, error_text):
        QMessageBox.critical(self,
            self.caption(),
            error_text,
            QMessageBox.Ok,
            QMessageBox.NoButton,
            QMessageBox.NoButton)
            
    def SuccessUI(self, text):
        QMessageBox.information(self,
                             self.caption(),
                             text,
                              QMessageBox.Ok,
                              QMessageBox.NoButton,
                              QMessageBox.NoButton)
            
        

    def plugin_download_callback(self, c, s, t):
        #pm.update(int(100*c*s/t), 
        #         utils.format_bytes(c*s))
        #print s
        pass

             
    def plugin_install_callback(self, s):
        print s
    
    
    def cancelPushButton_clicked(self):
        self.close()


    def __tr(self,s,c = None):
        return qApp.translate("PluginForm_base",s,c)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# (c) Copyright 2003-2007 Hewlett-Packard Development Company, L.P.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
# Author: Don Welch
#

from __future__ import division

__version__ = '2.1'
__mod__ = 'hp-scan'
__title__ = 'Scan Utility'
__doc__ = "SANE-based scan utility for HPLIP." 

# Std Lib
import sys
import os
import os.path
import getopt
import signal
import time
import socket
import operator

# Local
from base.g import *
from base import tui, device, module, utils
from prnt import cups


username = prop.username
res = 300
scan_mode = 'color'
tlx = None
tly = None
brx = None
bry = None
units = "mm"
output = ''
dest = []
email_from = ''
email_to = []
email_subject = 'hp-scan from %s' % socket.gethostname()
email_note = ''
fax = ''
resize = 100
contrast = 0
brightness = 0
#printer = ''
page_size = ''
size_desc = ''
page_units = 'mm'
valid_res = (75, 150, 300, 600, 1200, 2400, 4800)
default_res = 300
scanner_compression = 'JPEG'
adf = False

PAGE_SIZES = { # in mm
    '5x7' : (127, 178, "5x7 photo", 'in'),
    '4x6' : (102, 152, "4x6 photo", 'in'),
    '3x5' : (76, 127, "3x5 index card", 'in'),
    'a2_env' : (111, 146, "A2 Envelope", 'in'),
    'a3' : (297, 420, "A3", 'mm'),
    "a4" : (210, 297, "A4", 'mm'),
    "a5" : (148, 210, "A5", 'mm'),
    "a6" : (105, 148, "A6", 'mm'),
    "b4" : (257, 364, "B4", 'mm'),
    "b5" : (182, 257, "B5", 'mm'),
    "c6_env" : (114, 162, "C6 Envelope", 'in'),
    "dl_env" : (110, 220, "DL Envelope", 'in'),
    "exec" : (184, 267, "Executive", 'in'),
    "flsa" : (216, 330, "Flsa", 'mm'),
    "higaki" : (100, 148, "Hagaki", 'mm'),
    "japan_env_3" : (120, 235, "Japanese Envelope #3", 'mm'),
    "japan_env_4" : (90, 205, "Japanese Envelope #4", 'mm'),
    "legal" : (215, 356, "Legal", 'in'),
    "letter" : (215, 279, "Letter", 'in'),
    "no_10_env" : (105, 241, "Number 10 Envelope", 'in'),
    "oufufu-hagaki" : (148, 200, "Oufuku-Hagaki", 'mm'),
    "photo" : (102, 152, "Photo", 'in'),
    "super_b" : (330, 483, "Super B", 'in'),
    }


try:
    viewer = ''
    viewer_list = ['kview', 'display', 'gwenview', 'eog', 'kuickshow',]
    for v in viewer_list:
        vv = utils.which(v)
        if vv:
            viewer = os.path.join(vv, v)
            break


    editor = ''
    editor_list = ['kolourpaint', 'gimp', 'krita', 'cinepaint', 'mirage',]
    for e in editor_list:
        ee = utils.which(e)
        if ee:
            editor = os.path.join(ee, e)
            break

    pdf_viewer = ''
    pdf_viewer_list = ['kpdf', 'acroread', 'xpdf', 'evince',]
    for v in pdf_viewer_list:
        vv = utils.which(v)
        if vv:
            pdf_viewer = os.path.join(vv, v)
            break

    mod = module.Module(__mod__, __title__, __version__, __doc__, None,
                        (NON_INTERACTIVE_MODE,))

    mod.setUsage(module.USAGE_FLAG_DEVICE_ARGS,
        extra_options=[utils.USAGE_SPACE,
        ("[OPTIONS] (General)", "", "header", False),
        ("Scan destinations:", "-s<dest_list> or --dest=<dest_list>", "option", False),
        ("", "where <dest_list> is a comma separated list containing one or more of: 'file'\*, ", "option", False),
        ("", "'viewer', 'editor', 'pdf', 'fax', or 'print'. Use only commas between values, no spaces.", "option", False), 
        ("Scan mode:", "-m<mode> or --mode=<mode>. Where <mode> is 'color'\*, 'gray' or 'lineart'.", "option", False),
        ("Scanning resolution:", "-r<resolution_in_dpi> or --res=<resolution_in_dpi> or --resolution=<resolution_in_dpi>", "option", False),
        ("", "where <resolution_in_dpi> is %s (300 is default)." % ', '.join([str(x) for x in valid_res]), "option", False),
        ("Image resize:", "--resize=<scale_in_%> (min=1%, max=400%, default=100%)", "option", False),
        ("Image contrast:", "--contrast=<contrast>", "option", False),
        ("ADF mode (EXPERIMENTAL):", "--adf (Note, only PDF output is supported when using the ADF)", "option", False),
        utils.USAGE_SPACE,
        ("[OPTIONS] (Scan area)", "", "header", False),
        ("Specify the units for area/box measurements:", "-t<units> or --units=<units>", "option", False),
        ("", "where <units> is 'mm'\*, 'cm', 'in', 'px', or 'pt' ('mm' is default).", "option", False),
        ("Scan area:", "-a<tlx>,<tly>,<brx>,<bry> or --area=<tlx>,<tly>,<brx>,<bry>", "option", False),
        ("", "Coordinates are relative to the upper left corner of the scan area.", "option", False),
        ("", "Units for tlx, tly, brx, and bry are specified by -t/--units (default is 'mm').", "option", False),
        ("", "Use only commas between values, no spaces.", "option", False),
        ("Scan box:", "--box=<tlx>,<tly>,<width>,<height>", "option", False),
        ("", "tlx and tly coordinates are relative to the upper left corner of the scan area.", "option", False),
        ("", "Units for tlx, tly, width, and height are specified by -t/--units (default is 'mm').", "option", False),         
        ("", "Use only commas between values, no spaces.", "option", False),
        ("Top left x of the scan area:", "--tlx=<tlx>", "option", False),
        ("", "Coordinates are relative to the upper left corner of the scan area.", "option", False),
        ("", "Units are specified by -t/--units (default is 'mm').", "option", False),
        ("Top left y of the scan area:", "--tly=<tly>", "option", False),
        ("", "Coordinates are relative to the upper left corner of the scan area.", "option", False),
        ("", "Units are specified by -t/--units (default is 'mm').", "option", False),
        ("Bottom right x of the scan area:", "--brx=<brx>", "option", False),
        ("", "Coordinates are relative to the upper left corner of the scan area.", "option", False),
        ("", "Units are specified by -t/--units (default is 'mm').", "option", False),
        ("Bottom right y   of the scan area:", "--bry=<bry>", "option", False),
        ("", "Coordinates are relative to the upper left corner of the scan area.", "option", False),
        ("", "Units are specified by -t/--units (default is 'mm').", "option", False),
        ("Specify the scan area based on a paper size:", "--size=<paper size name>", "option", False),
        ("", "where <paper size name> is one of: %s" % ', '.join(PAGE_SIZES.keys()), "option", False), 
        utils.USAGE_SPACE,
        ("[OPTIONS] ('file' dest)", "", "header", False),
        ("Filename for 'file' destination:", "-o<file> or -f<file> or --file=<file> or --output=<file>", "option", False),
        utils.USAGE_SPACE,
        ("[OPTIONS] ('pdf' dest)", "", "header", False),
        ("PDF viewer application:", "--pdf=<pdf_viewer>", "option", False),
        utils.USAGE_SPACE,
        ("[OPTIONS] ('viewer' dest)", "", "header", False),
        ("Image viewer application:", "-v<viewer> or --viewer=<viewer>", "option", False),
        utils.USAGE_SPACE,
        ("[OPTIONS] ('editor' dest)", "", "header", False),
        ("Image editor application:", "-e<editor> or --editor=<editor>", "option", False),
        utils.USAGE_SPACE,
        ("[OPTIONS] ('email' dest)", "", "header", False),
        ("From: address for 'email' dest:", "--email-from=<email_from_address> (required for 'email' dest.)", "option", False),
        ("To: address for 'email' dest:", "--email-to=<email__to_address> (required for 'email' dest.)", "option", False),
        ("Email subject for 'email' dest:", '--email-subject="<subject>" or --subject="<subject>"', "option", False),
        ("", 'Use double quotes (") around the subject if it contains space characters.', "option", False),
        ("Note or message for the 'email' dest:", '--email-msg="<msg>" or --email-note="<note>"', "option", False),
        ("", 'Use double quotes (") around the note/message if it contains space characters.', "option", False),
        utils.USAGE_SPACE,
        ("[OPTIONS] ('fax' dest)", "", "header", False),
        ("Fax queue/printer:", "--fax=<fax_printer_name>", "option", False),
        utils.USAGE_SPACE,
        ("[OPTIONS] ('printer' dest)", "", "header", False),
        ("Printer queue/printer:", "--printer=<printer_name>", "option", False),
        utils.USAGE_SPACE,
        ("[OPTIONS] (advanced)", "", "header", False),
        ("Set the scanner compression mode:", "-x<mode> or --compression=<mode>, <mode>='raw', 'none' or 'jpeg' ('jpeg' is default) ('raw' and 'none' are equivalent)", "option", False),],
        see_also_list=[])
              
    opts, device_uri, printer_name, mode, ui_toolkit, lang = \
        mod.parseStdOpts('s:m:r:c:t:a:b:o:v:f:c:x:',
                         ['dest=', 'mode=', 'res=', 'resolution=',
                          'resize=', 'contrast=', 'adf', 'unit=',
                          'units=', 'area=', 'box=', 'tlx=',
                          'tly=', 'brx=', 'bry=', 'size=',
                          'file=', 'output=', 'pdf=', 'viewer=',
                          'email-from=', 'from=', 'email-to=',
                          'to=', 'email-msg=', 'msg=', 'fax=',
                          'printer=', 'compression=' , 'raw',
                          'jpeg', 'color', 'lineart', 'colour',
                          'bw', 'gray', 'grayscale', 'grey',
                          'greyscale', 'email-subject=',
                          'subject=', 'to=', 'from=', 'jpg',
                          'grey-scale', 'gray-scale', 'about=',
                         ])
                          
 

    device_uri = mod.getDeviceUri(device_uri, printer_name, 
        back_end_filter=['hpaio'], filter={'scan-type': (operator.gt, 0)})

    for o, a in opts:
        if o in ('-x', '--compression'):
            a = a.strip().lower()

            if a in ('jpeg', 'jpg'):
                scanner_compression = 'JPEG'

            elif a in ('raw', 'none'):
                scanner_compression = 'None'

            else:
                log.error("Invalid compression value. Valid values are 'jpeg', 'raw', and 'none'.")
                log.error("Using default value of 'jpeg'.")
                scanner_compression = 'JPEG'

        elif o == 'raw':
            scanner_compression = 'None'

        elif o == 'jpeg':
            scanner_compression = 'JPEG'

        elif o in ('--color', '--colour'):
            scan_mode = 'color'
                
        elif o in ('--lineart', '--line-art', '--bw'):
            scan_mode = 'lineart'
            
        elif o in ('--gray', '--grayscale', '--gray-scale', '--grey', '--greyscale', '--grey-scale'):
            scan_mode = 'gray'
            
        elif o in ('-m', '--mode'):
            a = a.strip().lower()

            if a in ('color', 'colour'):
                scan_mode = 'color'

            elif a in ('lineart', 'bw', 'b&w'):
                scan_mode = 'lineart'

            elif a in ('gray', 'grayscale', 'grey', 'greyscale'):
                scan_mode = 'gray'

            else:
                log.error("Invalid mode. Using default of 'color'.")
                log.error("Valid modes are 'color', 'lineart', or 'gray'.")
                scan_mode = 'color'

        elif o in ('--res', '--resolution', '-r'):
            try:
                r = int(a.strip())
            except ValueError:
                log.error("Invalid resolution. Using default of %s dpi." % default_res)
                log.error("Valid resolutions are %s dpi." % ', '.join([str(x) for x in valid_res]))
                res = default_res
            else:
                if r in valid_res: 
                    res = r
                else:
                    res = valid_res[0]
                    min_dist = sys.maxint
                    for x in valid_res: 
                        if abs(r-x) < min_dist:
                            min_dist = abs(r-x)
                            res = x

                    log.warn("Invalid resolution. Using closest valid resolution of %d dpi" % res)
                    log.error("Valid resolutions are %s dpi." % ', '.join([str(x) for x in valid_res]))

        elif o in ('-t', '--units', '--unit'):
            a = a.strip().lower()

            if a in ('in', 'inch', 'inches'):
                units = 'in'

            elif a in ('mm', 'milimeter', 'milimeters', 'millimetre', 'millimetres'):
                units = 'mm'

            elif a in ('cm', 'centimeter', 'centimeters', 'centimetre', 'centimetres'):
                units = 'cm'

            elif a in ('px', 'pixel', 'pixels', 'pel', 'pels'):
                units = 'px'

            elif a in ('pt', 'point', 'points', 'pts'):
                units = 'pt'

            else:
                log.error("Invalid units. Using default of 'mm'.")
                units = 'mm'

        elif o == '--tlx':
            a = a.strip().lower()
            try:
                f = float(a)
            except ValueError:
                log.error("Invalid value for tlx.")
            else:
                tlx = f

        elif o == '--tly':
            a = a.strip().lower()
            try:
                f = float(a)
            except ValueError:
                log.error("Invalid value for tly.")
            else:
                tly = f

        elif o == '--brx':
            a = a.strip().lower()
            try:
                f = float(a)
            except ValueError:
                log.error("Invalid value for brx.")
            else:
                brx = f

        elif o == '--bry':
            a = a.strip().lower()
            try:
                f = float(a)
            except ValueError:
                log.error("Invalid value for bry.")
            else:
                bry = f

        elif o in ('-a', '--area'): # tlx, tly, brx, bry
            a = a.strip().lower()
            try:
                tlx, tly, brx, bry = a.split(',')[:4]
            except ValueError:
                log.error("Invalid scan area. Using defaults.")
            else:
                try:
                    tlx = float(tlx)
                except ValueError:
                    log.error("Invalid value for tlx. Using defaults.")
                    tlx = None

                try:
                    tly = float(tly)
                except ValueError:
                    log.error("Invalid value for tly. Using defaults.")
                    tly = None

                try:
                    brx = float(brx)
                except ValueError:
                    log.error("Invalid value for brx. Using defaults.")
                    brx = None

                try:
                    bry = float(bry)
                except ValueError:
                    log.error("Invalid value for bry. Using defaults.")
                    bry = None

        elif o in ('-b', '--box'): # tlx, tly, w, h
            a = a.strip().lower()
            try:
                tlx, tly, width, height = a.split(',')[:4]
            except ValueError:
                log.error("Invalid scan area. Using defaults.")
            else:
                try:
                    tlx = float(tlx)
                except ValueError:
                    log.error("Invalid value for tlx. Using defaults.")
                    tlx = None

                try:
                    tly = float(tly)
                except ValueError:
                    log.error("Invalid value for tly. Using defaults.")
                    tly = None

                if tlx is not None:
                    try:
                        brx = float(width) + tlx
                    except ValueError:
                        log.error("Invalid value for width. Using defaults.")
                        brx = None
                else:
                    log.error("Cannot calculate brx since tlx is invalid. Using defaults.")
                    brx = None

                if tly is not None:
                    try:
                        bry = float(height) + tly
                    except ValueError:
                        log.error("Invalid value for height. Using defaults.")
                        bry = None    
                else:
                    log.error("Cannot calculate bry since tly is invalid. Using defaults.")
                    bry = None

        elif o == '--size':
            size = a.strip().lower()
            if size in PAGE_SIZES:
                brx, bry, size_desc, page_units = PAGE_SIZES[size]
                tlx, tly = 0, 0
                page_size = size
            else:
                log.error("Invalid page size. Valid page sizes are: %s" % ', '.join(PAGE_SIZES.keys()))
                log.error("Using defaults.")

        elif o in ('-o', '--output', '-f', '--file'):
            output = os.path.abspath(os.path.normpath(os.path.expanduser(a.strip())))

            try:
                ext = os.path.splitext(output)[1]
            except IndexError:
                log.error("Invalid filename extension.")
                output = ''
                if 'file' in dest:
                    dest.remove('file')
            else:
                if ext.lower() not in ('.jpg', '.png'):
                    log.error("Only JPG (.jpg) and PNG (.png) output files are supported.")
                    output = ''
                    if 'file' in dest:
                        dest.remove('file')
                else:
                    if os.path.exists(output):
                        log.warn("Output file '%s' exists. File will be overwritten." % output)

                    if 'file' not in dest:
                        dest.append('file')

        elif o in ('-s', '--dest', '--destination'):
            a = a.strip().lower().split(',')
            for aa in a:
                aa = aa.strip()
                if aa in ('file', 'fax', 'viewer', 'editor', 'printer', 'print', 'email', 'pdf') \
                    and aa not in dest:
                    if aa == 'print': aa = 'printer'
                    dest.append(aa)

        elif o in ('-v', '--viewer'):
            a = a.strip()
            b = utils.which(a)
            if not b:
                log.error("Viewer application not found.")
            else:
                viewer = os.path.join(b, a)
                if 'viewer' not in dest:
                    dest.append('viewer')

        elif o in ('-e', '--editor'):
            a = a.strip()
            b = utils.which(a)
            if not b:
                log.error("Editor application not found.")
            else:
                editor = os.path.join(b, a)
                if 'editor' not in dest:
                    dest.append('editor')

        elif o == '--pdf':
            a = a.strip()
            b = utils.which(a)
            if not b:
                log.error("PDF viewer application not found.")
            else:
                pdf_viewer = os.path.join(b, a)
                if 'pdf' not in dest:
                    dest.append('pdf')

#        elif o == '--fax':
#            pp = a.strip()
#            from prnt import cups
#            printer_list = cups.getPrinters()
#            found = False
#            for p in printer_list:
#                if p.name == pp:
#                    found = True
#                    fax = pp
#                    break
#
#            if found: 
#                if 'fax' not in dest:
#                    dest.append('fax')
#            else:
#                log.error("Unknown/invalid fax name: %s" % pp)

        elif o in ('--email-to', '--to'):
            email_to = a.split(',')
            if 'email' not in dest:
                dest.append('email')

        elif o in ('--email-from', '--from'):
            email_from = a
            if 'email' not in dest:
                dest.append('email')

        elif o in ('--email-subject', '--subject', '--about'):
            email_subject = a
            if 'email' not in dest:
                dest.append('email')

        elif o in ('--email-note', '--email-msg', '--msg', '--message', '--note', '--notes'):
            email_note = a
            if 'email' not in dest:
                dest.append('email')

        elif o == '--resize':
            a = a.replace("%", "")
            try:
                resize = int(a)
            except ValueError:
                resize = 100
                log.error("Invalid resize value. Using default of 100%.")

        elif o in ('-b', '--brightness'):
            pass

        elif o in ('-c', '--contrast'):
            try:
                contrast = int(a.strip())
            except ValueError:
                log.error("Invalid contrast value. Using default of 100.")
                contrast = 100

        elif o == '--adf':
            adf = True
            output_type = 'pdf'


    if printer_name is not None and \
        device.getDeviceURIByPrinterName(printer_name) is not None and \
        'printer' not in dest:

        dest.append('printer')        

#    if 'printer' in dest and not printer:
#        from prnt import cups
#        printer = cups.getDefaultPrinter()
#
#        if printer is not None:
#            log.warn("Print destination enabled with no printer specified.")
#            log.warn("Using CUPS default printer '%s'." % printer)
#        else:
#            log.error("Print destination enabled with no printer specified.")
#            log.error("No CUPS default printer found. Disabling 'print' destination.")
#            dest.remove("printer")

    if 'fax' in dest and 'file' not in dest:
        log.error("Fax destination not implemented. Adding 'file' destination. Use output file to fax.")
        dest.append('file')

    if not dest:
        log.warn("No destinations specified. Adding 'file' destination by default.")
        dest.append('file')

    if 'file' in dest and not output:
        log.warn("File destination enabled with no output file specified.")

        if adf:
            log.info("Setting output format to PDF for ADF mode.")
            output = utils.createSequencedFilename("hpscan", ".pdf")
            output_type = 'pdf'
        else:
            if scan_mode == 'gray':
                log.info("Setting output format to PNG for greyscale mode.")
                output = utils.createSequencedFilename("hpscan", ".png")
                output_type = 'png'
            else:
                log.info("Setting output format to JPEG for color/lineart mode.")
                output = utils.createSequencedFilename("hpscan", ".jpg")
                output_type = 'jpeg'

        log.warn("Defaulting to '%s'." % output)

    else:
        try:
            output_type = os.path.splitext(output)[1].lower()[1:]
            if output_type == 'jpg':
                output_type = 'jpeg'
        except IndexError:
            output_type = ''

    if output_type and output_type not in ('jpeg', 'png', 'pdf'):
        log.error("Invalid output file format. File formats must be 'jpeg', 'png', or 'pdf'.")
        sys.exit(1)

    #sys.exit(0)
    ##if scan_mode == 'gray' and output_type and output_type != 'png':
    ##    log.error("Grayscale scans must be saved in PNG file format. To save in other formats, set the 'editor' destination and save the image from the editor in the desired format.")
    ##    sys.exit(1)

    if adf and output_type and output_type != 'pdf':
        log.error("ADF scans must be saved in PDF file format.")
        sys.exit(1)

    if 'email' in dest and (not email_from or not email_to):
        log.error("Email specified, but email to and/or email from address(es) were not specified.")
        log.error("Disabling 'email' destination.")
        dest.remove("email")

    if page_size:
        units = 'mm'

    if units == 'in':
        if tlx is not None: tlx = tlx * 25.4
        if tly is not None: tly = tly * 25.4
        if brx is not None: brx = brx * 25.4
        if bry is not None: bry = bry * 25.4

    elif units == 'cm':
        if tlx is not None: tlx = tlx * 10.0
        if tly is not None: tly = tly * 10.0
        if brx is not None: brx = brx * 10.0
        if bry is not None: bry = bry * 10.0

    elif units == 'pt':
        if tlx is not None: tlx = tlx * 0.3528
        if tly is not None: tly = tly * 0.3528
        if brx is not None: brx = brx * 0.3528
        if bry is not None: bry = bry * 0.3528

    elif units == 'px':
        log.warn("Units set to pixels. Using resolution of %ddpi for area calculations." % res)
        if tlx is not None: tlx = tlx / res * 25.4
        if tly is not None: tly = tly / res * 25.4
        if brx is not None: brx = brx / res * 25.4
        if bry is not None: bry = bry / res * 25.4

    if tlx is not None and brx is not None and tlx >= brx:
        log.error("Invalid values for tlx (%d) and brx (%d) (tlx>=brx). Using defaults." % (tlx, brx))
        tlx = brx = None

    if tly is not None and bry is not None and tly >= bry:
        log.error("Invalid values for tly (%d) and bry (%d) (tly>=bry). Using defaults." % (tly, bry))
        tly = bry = None

    if not prop.scan_build:
        log.error("Scanning disabled in build. Exiting")
        sys.exit(1)

    if mode == GUI_MODE:
        log.error("GUI mode is not implemented yet. Please use -n. Refer to 'hp-scan -h' for help.")
        sys.exit(1)

#        if not prop.gui_build:
#            log.warn("GUI mode disabled in build. Reverting to interactive mode.")
#            mode = NON_INTERACTIVE_MODE
#
#        elif not os.getenv('DISPLAY'):
#            log.warn("No display found. Reverting to interactive mode.")
#            mode = NON_INTERACTIVE_MODE
#
#        elif not utils.checkPyQtImport():
#            log.warn("PyQt init failed. Reverting to interactive mode.")
#            mode = NON_INTERACTIVE_MODE

#    if mode == GUI_MODE:
#        app = None
#        sendfax = None
#
#        from qt import *
#
#        # UI Forms
#        from ui.scanform import ScanForm
#
#        # create the main application object
#        app = QApplication(sys.argv)
#
#        if loc is None:
#            loc = user_cfg.ui.get("loc", "system")
#            if loc.lower() == 'system':
#                loc = str(QTextCodec.locale())
#                log.debug("Using system locale: %s" % loc)
#
#        if loc.lower() != 'c':
#            e = 'utf8'
#            try:
#                l, x = loc.split('.')
#                loc = '.'.join([l, e])
#            except ValueError:
#                l = loc
#                loc = '.'.join([loc, e])
#
#            log.debug("Trying to load .qm file for %s locale." % loc)
#            trans = QTranslator(None)
#
#            qm_file = 'hplip_%s.qm' % l
#            log.debug("Name of .qm file: %s" % qm_file)
#            loaded = trans.load(qm_file, prop.localization_dir)
#
#            if loaded:
#                app.installTranslator(trans)
#            else:
#                loc = 'c'
#
#        if loc == 'c':
#            log.debug("Using default 'C' locale")
#        else:
#            log.debug("Using locale: %s" % loc)
#            QLocale.setDefault(QLocale(loc))
#            prop.locale = loc
#            try:
#                locale.setlocale(locale.LC_ALL, locale.normalize(loc))
#            except locale.Error:
#                pass
#
#        scanui = ScanForm(device_uri, printer_name, args) 
#
#        app.setMainWidget(scanui)
#
#        pid = os.getpid()
#        log.debug('pid=%d' % pid)
#
#        scanui.show()
#
#        signal.signal(signal.SIGPIPE, signal.SIG_IGN)
#
#        try:
#            log.debug("Starting GUI loop...")
#            app.exec_loop()
#        except KeyboardInterrupt:
#            pass
#        #except:
#        #    log.exception()

    if 1:
        #else: # NON_INTERACTIVE_MODE
        import Queue
        from scan import sane
        import scanext
        import cStringIO

        try:
            import subprocess
        except ImportError:
            # Pre-2.4 Python
            from base import subproc as subprocess

        try:
            import Image
        except ImportError:
            log.error("%s requires the Python Imaging Library (PIL). Exiting." % __mod__)
            sys.exit(1)

    ##    if output_type == 'pdf':
    ##        try:
    ##            from reportlab.pdfgen import canvas
    ##        except ImportError:
    ##            log.error("PDF output requires ReportLab.")
    ##            sys.exit(1)

        sane.init()
        devices = sane.getDevices()
        
        # Make sure SANE backend sees the device...
        for d, mfg, mdl, t in devices:
            if d == device_uri:
                break
        else:
            log.error("Unable to locate device %s using SANE backend hpaio:. Please check HPLIP installation." % device_uri)
            sys.exit(1)
            
        log.info(log.bold("Using device %s" % device_uri))
        log.info("Opening connection to device...")

        try:
            device = sane.openDevice(device_uri)
        except scanext.error, e:
            log.error(e)
            sys.exit(1)

        #print device.options
        #sys.exit(0)

        tlx = device.getOptionObj('tl-x').limitAndSet(tlx)
        tly = device.getOptionObj('tl-y').limitAndSet(tly)
        brx = device.getOptionObj('br-x').limitAndSet(brx)
        bry = device.getOptionObj('br-y').limitAndSet(bry)

        scan_area = (brx - tlx) * (bry - tly) # mm^2
        scan_px = scan_area * res * res / 645.16 # res is in DPI

        if scan_mode == 'color':
            scan_size = scan_px * 3 # 3 bytes/px
        else:
            scan_size = scan_px # 1 byte/px

        if scan_size > 52428800: # 50MB
            if res > 600:
                log.warn("Using resolutions greater than 600 dpi will cause very large files to be created.")
            else:
                log.warn("The scan current parameters will cause very large files to be created.")

            log.warn("This can cause the scan to take a long time to complete and may cause your system to slow down.")
            log.warn("Approx. number of bytes to read from scanner: %s" % utils.format_bytes(scan_size, True))

        res = device.getOptionObj('resolution').limitAndSet(res)

        device.setOption('compression', scanner_compression)

        if brx - tlx <= 0.0 or bry - tly <= 0.0:
            log.error("Invalid scan area (width or height is negative).")
            sys.exit(1)

        log.info("")
        log.info("Resolution: %ddpi" % res)
        log.info("Mode: %s" % scan_mode)
        log.info("Compression: %s" % scanner_compression)
        log.info("Scan area (mm):")
        log.info("  Top left (x,y): (%fmm, %fmm)" % (tlx, tly))
        log.info("  Bottom right (x,y): (%fmm, %fmm)" % (brx, bry))
        log.info("  Width: %fmm" % (brx - tlx))
        log.info("  Height: %fmm" % (bry - tly))

        if page_size:
            units = page_units # for display purposes only
            log.info("Page size: %s" % size_desc)
            if units != 'mm':
                log.note("This scan area below in '%s' units may not be exact due to rounding errors." % units)

        if units == 'in':
            log.info("Scan area (in):")
            log.info("  Top left (x,y): (%fin, %fin)" % (tlx/25.4, tly/25.4))
            log.info("  Bottom right (x,y): (%fin, %fin)" % (brx/25.4, bry/25.4))
            log.info("  Width: %fin" % ((brx - tlx)/25.4))
            log.info("  Height: %fin" % ((bry - tly)/25.4))

        elif units == 'cm':
            log.info("Scan area (cm):")
            log.info("  Top left (x,y): (%fcm, %fcm)" % (tlx/10.0, tly/10.0))
            log.info("  Bottom right (x,y): (%fcm, %fcm)" % (brx/10.0, bry/10.0))
            log.info("  Width: %fcm" % ((brx - tlx)/10.0))
            log.info("  Height: %fcm" % ((bry - tly)/10.0))

        elif units == 'px':
            log.info("Scan area (px @ %ddpi):" % res)
            log.info("  Top left (x,y): (%fpx, %fpx)" % (tlx*res/25.4, tly*res/25.4))
            log.info("  Bottom right (x,y): (%fpx, %fpx)" % (brx*res/25.4, bry*res/25.4))
            log.info("  Width: %fpx" % ((brx - tlx)*res/25.4))
            log.info("  Height: %fpx" % ((bry - tly)*res/25.4))

        elif units == 'pt':
            log.info("Scan area (pt):")
            log.info("  Top left (x,y): (%fpt, %fpt)" % (tlx/0.3528, tly/0.3528))
            log.info("  Bottom right (x,y): (%fpt, %fpt)" % (brx/0.3528, bry/0.3528))
            log.info("  Width: %fpt" % ((brx - tlx)/0.3528))
            log.info("  Height: %fpt" % ((bry - tly)/0.3528))

        log.info("Destination(s): %s" % ', '.join(dest))

        if 'file' in dest:
            log.info("Output file: %s" % output)

        update_queue = Queue.Queue()
        event_queue = Queue.Queue()

        device.setOption("mode", scan_mode)
        device.setOption("resolution", res)

        if adf:
            try:
                device.setOption("source", "ADF")
                device.setOption("batch-scan", True)
            except scanext.error:
                log.error("Failed to set ADF mode. Does this device support ADF? Disabling ADF mode.")
                adf = False

        if not adf:
            try:
                #device.setOption("source", "Auto")
                device.setOption("batch-scan", False)
            except scanext.error:
                log.debug("Error setting source or batch-scan option (this is probably OK).")

        log.info("\nWarming up...")

        no_docs = False
        page = 1
        adf_page_files = []
        #adf_pages = []

        cleanup_spinner()
        log.info("")

        try:
            while True:
                if adf:
                    log.info("\nPage %d: Scanning..." % page)
                else:
                    log.info("\nScanning...")

                bytes_read = 0

                try:
                    try:
                        ok, expected_bytes, status = device.startScan("RGBA", update_queue, event_queue)
                        # Note: On some scanners (Marvell) expected_bytes will be < 0 (if lines == -1)
                        log.debug("expected_bytes = %d" % expected_bytes)
                    except scanext.error, e:
                        log.error(e)
                        sys.exit(1)
                    except KeyboardInterrupt:
                        log.error("Aborted.")
                        device.cancelScan()
                        sys.exit(1) 

                    if adf and status == scanext.SANE_STATUS_NO_DOCS:
                        if page-1 == 0:
                            log.error("No document(s). Please load documents and try again.")
                            sys.exit(0)
                        else:    
                            log.info("Out of documents. Scanned %d pages total." % (page-1))
                            no_docs = True
                            break

                    if expected_bytes > 0:
                        if adf:
                            log.info("Expecting to read %s from scanner (per page)." % utils.format_bytes(expected_bytes))
                        else:
                            log.info("Expecting to read %s from scanner." % utils.format_bytes(expected_bytes))

                    device.waitForScanActive()

                    pm = tui.ProgressMeter("Reading data:")

                    while device.isScanActive():
                        while update_queue.qsize():
                            try:
                                status, bytes_read = update_queue.get(0)

                                #if log.get_level() >= log.LOG_LEVEL_INFO:
                                if not log.is_debug():
                                    if expected_bytes > 0:
                                        pm.update(int(100*bytes_read/expected_bytes), 
                                            utils.format_bytes(bytes_read))
                                    else:
                                        pm.update(0, 
                                            utils.format_bytes(bytes_read))
                                        

    ##                            if status == scanext.SANE_STATUS_EOF:
    ##                                log.debug("EOF")
    ##                            elif status == scanext.SANE_STATUS_NO_DOCS:
    ##                                log.debug("NO DOCS")
    ##                                no_docs = True
                                #elif status != scanext.SANE_STATUS_GOOD:
                                if status != scanext.SANE_STATUS_GOOD:
                                    log.error("SANE error %d" % status)
                                    sys.exit(1)

                            except Queue.Empty:
                                break


                        time.sleep(0.5)

                except KeyboardInterrupt:
                    log.error("Aborted.")
                    device.cancelScan()
                    sys.exit(1)

                # Make sure queue is cleared out...
                while update_queue.qsize():
                    status, bytes_read = update_queue.get(0)

                    if not log.is_debug():
                        if expected_bytes > 0:
                            pm.update(int(100*bytes_read/expected_bytes), 
                                utils.format_bytes(bytes_read))
                        else:
                            pm.update(0, 
                                utils.format_bytes(bytes_read))
                            
    ##                if status == scanext.SANE_STATUS_EOF:
    ##                    log.debug("EOF")
    ##                elif status == scanext.SANE_STATUS_NO_DOCS:
    ##                    log.debug("NO DOCS")
    ##                    no_docs = True

                    if status != scanext.SANE_STATUS_GOOD:
                        log.error("SANE error %d" % status)
                        sys.exit(1)

                log.info("")

                if bytes_read:
                    log.info("Read %s from scanner." % utils.format_bytes(bytes_read))

                    buffer, format, format_name, pixels_per_line, \
                        lines, depth, bytes_per_line, pad_bytes, total_read = device.getScan()
                        
                    log.debug("PPL=%d lines=%d depth=%d BPL=%d pad=%d total=%d" % 
                        (pixels_per_line, lines, depth, bytes_per_line, pad_bytes, total_read))
                        
                    if lines == -1:
                        lines = int(total_read / bytes_per_line)

                    if scan_mode in ('color', 'gray'):
                        try:
                            im = Image.frombuffer('RGBA', (pixels_per_line, lines), buffer.read(), 
                                'raw', 'RGBA', 0, 1)
                        except ValueError:
                            log.error("Did not read enough data from scanner (I/O Error?)")
                            sys.exit(1)

    ##                elif scan_mode == 'gray':
    ##                    im = Image.frombuffer('RGBA', (pixels_per_line, lines), buffer.read(), 
    ##                        'raw', 'RGBA', 0, 1).convert('P')

                    elif scan_mode == 'lineart':
                        try:
                            im = Image.frombuffer('RGBA', (pixels_per_line, lines), buffer.read(), 
                                'raw', 'RGBA', 0, 1).convert('L')
                        except ValueError:
                            log.error("Did not read enough data from scanner (I/O Error?)")
                            sys.exit(1)

                    if adf:
                        temp_output = utils.createSequencedFilename("hpscan_pg%d_" % page, ".png")
                        adf_page_files.append(temp_output)
                        im.save(temp_output)  
                        log.debug("Saved page %d to file %s" % (page, temp_output))
                        #adf_pages.append(im)
                else:
                    log.error("No data read.")
                    sys.exit(1)

                if not adf or (adf and no_docs):
                    break

                page += 1

        finally:
            log.info("Closing device.")
            device.cancelScan()
            #print "0"
            #device.freeScan()
            #sane.deInit()

        #if no_docs:
        #    sys.exit(0)

        #print "1"
        if adf:
            try:
                from reportlab.pdfgen import canvas
            except ImportError:
                log.error("PDF output requires ReportLab.")
                sys.exit(1)
            #print "2"
            #print canvas
            #print canvas.Canvas

            tlx_max = device.getOptionObj('tl-x').constraint[1]
            bry_max = device.getOptionObj('br-y').constraint[1]

            if not output:
                output = utils.createSequencedFilename("hpscan", ".pdf")

            c = canvas.Canvas(output, (tlx_max/0.3528, bry_max/0.3528))
            #print c

            #for p in adf_page_files:
            for p in adf_page_files:
                log.info("Processing page %s..." % p)

                image = Image.open(p)
                #print image

                try:
                    c.drawInlineImage(image, (tlx/0.3528), ((bry_max/0.3528)-(bry/0.3528)), 

                    #c.drawInlineImage(image, 0, bry/0.3528,
                        width=None,height=None)
                except NameError:
                    log.error("A problem has occurred with PDF generation. This is a known bug in ReportLab. Please update your install of ReportLab to version 2.0 or greater.")
                    sys.exit(1)

                c.showPage()

            log.info("Saving to file %s" % output)
            c.save()
            log.info("Viewing PDF file in %s" % pdf_viewer)
            os.system("%s %s &" % (pdf_viewer, output))            


            sys.exit(0)

        if resize != 100:
            if resize < 1 or resize > 400:
                log.error("Resize parameter is incorrect. Resize must be 0% < resize < 400%.")
                log.error("Using resize value of 100%.")
            else:
                new_w = pixels_per_line * resize / 100
                new_h = lines * resize / 100
                log.info("Resizing from %dx%d to %dx%d..." % (pixels_per_line, lines, new_w, new_h))
                im = im.resize((new_w, new_h), Image.ANTIALIAS)

        file_saved = False
        if 'file' in dest:
            log.info("\nOutputting to destination 'file':")
            log.info("Saving to file %s" % output)

            try:
                im.save(output)
            except IOError, e:
                log.error("Error saving file: %s (I/O)" % e)
                try:
                    os.remove(output)
                except OSError:
                    pass
                sys.exit(1)
            except ValueError, e:
                log.error("Error saving file: %s (PIL)" % e)
                try:
                    os.remove(output)
                except OSError:
                    pass
                sys.exit(1)

            file_saved = True
            dest.remove("file")

        temp_saved = False
        if ('editor' in dest or 'viewer' in dest or 'email' in dest or 'printer' in dest) \
            and not file_saved:

            output_fd, output = utils.make_temp_file(suffix='.png')
            try:
                im.save(output)
            except IOError, e:
                log.error("Error saving temporary file: %s" % e)

                try:
                    os.remove(output)
                except OSError:
                    pass

                sys.exit(1)

            os.close(output_fd) 
            temp_saved = True      

        for d in dest:
            log.info("\nSending to destination '%s':" % d)

            if d == 'fax':
                log.error("fax: Not implemented yet.")

            elif d == 'pdf':
                try:
                    from reportlab.pdfgen import canvas
                except ImportError:
                    log.error("PDF output requires ReportLab.")
                    continue

                tlx_max = device.getOptionObj('tl-x').constraint[1]
                bry_max = device.getOptionObj('br-y').constraint[1]

                pdf_output = utils.createSequencedFilename("hpscan", ".pdf")
                c = canvas.Canvas(pdf_output, (tlx_max/0.3528, bry_max/0.3528))

                try:
                    c.drawInlineImage(im, (tlx/0.3528), ((bry_max/0.3528)-(bry/0.3528)), 
                        width=None,height=None)
                except NameError:
                    log.error("A problem has occurred with PDF generation. This is a known bug in ReportLab. Please update your install of ReportLab to version 2.0 or greater.")
                    continue

                c.showPage()
                log.info("Saving to file %s" % pdf_output)
                c.save()
                log.info("Viewing PDF file in %s" % pdf_viewer)
                os.system("%s %s &" % (pdf_viewer, pdf_output))

            elif d == 'printer':
                hp_print = utils.which("hp-print")
                if hp_print:
                    cmd = 'hp-print %s &' % output
                else:
                    cmd = "python ./print.py %s &" % output

                os.system(cmd)

            elif d == 'email':
                try:
                    from email.mime.image import MIMEImage
                    from email.mime.multipart import MIMEMultipart
                    from email.mime.text import MIMEText
                except ImportError:
                    try:
                        from email.MIMEImage import MIMEImage
                        from email.MIMEMultipart import MIMEMultipart
                        from email.MIMEText import MIMEText
                    except ImportError:
                        log.error("hp-scan email destination requires Python 2.2+.")
                        continue

                msg = MIMEMultipart()
                msg['Subject'] = email_subject
                msg['From'] = email_from
                msg['To'] = ','.join(email_to)
                msg.preamble = 'Scanned using hp-scan'

                if email_note:
                    txt = MIMEText(email_note)
                    msg.attach(txt)

                if file_saved:
                    txt = MIMEText("attached: %s: %dx%d %s PNG image." % 
                        (os.path.basename(output), pixels_per_line, lines, scan_mode))
                else:
                    txt = MIMEText("attached: %dx%d %s PNG image." % (pixels_per_line, lines, scan_mode))

                msg.attach(txt) 

                fp = open(output, 'r')
                img = MIMEImage(fp.read())
                fp.close()

                if file_saved:
                    img.add_header('Content-Disposition', 'attachment', filename=os.path.basename(output))

                msg.attach(img)

                sendmail = utils.which("sendmail")

                if sendmail:
                    sendmail = os.path.join(sendmail, 'sendmail')
                    cmd = [sendmail,'-t','-r',email_from]

                    log.debug(repr(cmd))
                    err = None
                    try:
                        sp = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
                        std_out, std_err = sp.communicate(msg.as_string())
                        if std_err != '':
                            err = std_err
                    except OSError, e:
                        err = str(e)
                    cleanup_spinner()

                    if err:
                        log.error(repr(err))

                else:
                    log.error("Mail send failed. 'sendmail' not found.")

            elif d == 'viewer':
                if viewer:
                    log.info("Viewing file in %s" % viewer)
                    os.system("%s %s &" % (viewer, output))
                else:
                    log.error("Viewer not found.")

            elif d == 'editor':
                if editor:
                    log.info("Editing file in %s" % editor)
                    os.system("%s %s &" % (editor, output))
                else:
                    log.error("Editor not found.")

        device.freeScan()
        sane.deInit()    


except KeyboardInterrupt:
    log.error("User exit")

log.info("")
log.info("Done.")

